@@ -1,5 +1,5 @@
 /*
- * $Id: internal.h,v 1.15.2.1 2008/09/02 21:12:59 sfeam Exp $
+ * $Id: internal.h,v 1.19.2.2 2010/09/19 02:31:57 sfeam Exp $
  */
 
 /* GNUPLOT - internal.h */
@@ -44,6 +44,7 @@
 #include "eval.h"
 
 /* Prototypes from file "internal.c" */
+void eval_reset_after_error __PROTO((void));
 
 /* the basic operators of our stack machine for function evaluation: */
 void f_push __PROTO((union argument *x));
@@ -51,6 +52,7 @@ void f_pushc __PROTO((union argument *x));
 void f_pushd1 __PROTO((union argument *x));
 void f_pushd2 __PROTO((union argument *x));
 void f_pushd __PROTO((union argument *x));
+void f_pop __PROTO((union argument *x));
 void f_call __PROTO((union argument *x));
 void f_calln __PROTO((union argument *x));
 void f_lnot __PROTO((union argument *x));
@@ -75,7 +77,6 @@ void f_mod __PROTO((union argument *x));
 void f_power __PROTO((union argument *x));
 void f_factorial __PROTO((union argument *x));
 
-#ifdef GP_STRING_VARS
 void f_concatenate __PROTO((union argument *x));
 void f_eqs __PROTO((union argument *x));
 void f_nes __PROTO((union argument *x));
@@ -88,6 +89,7 @@ void f_system __PROTO((union argument *x));
 void f_words __PROTO((union argument *x));
 void f_strftime __PROTO((union argument *x));
 void f_strptime __PROTO((union argument *x));
-#endif
+void f_assign __PROTO((union argument *x));
+void f_value __PROTO((union argument *x));
 
 #endif /* GNUPLOT_INTERNAL_H */
