@@ -1,5 +1,5 @@
 /*
- * $Id: pm3d.h,v 1.26 2006/06/15 15:42:33 sfeam Exp $
+ * $Id: pm3d.h,v 1.27 2008/03/13 21:00:37 sfeam Exp $
  */
 
 /* GNUPLOT - pm3d.h */
@@ -34,12 +34,6 @@
   Global options for pm3d algorithm (to be accessed by set / show)
 ****/
 
-/* HBB NEW 20040311: setting this to 0 removes the pm3d option
- * {transparent|solid} --- it's a duplicate of 'set grid front' */
-#ifndef PM3D_HAVE_SOLID
-# define PM3D_HAVE_SOLID 0
-#endif
-
 /*
   where to plot pm3d: base or top (color map) or surface (color surface)
   The string pm3d.where can be any combination of the #defines below.
@@ -113,10 +107,6 @@ typedef struct {
   char clip;		/* 1in, 4in */
   int hidden3d_tag;     /* if this is > 0, pm3d hidden lines are drawn with
 			   this linestyle (which must naturally present). */
-#if PM3D_HAVE_SOLID
-  int solid;            /* if this is != 0, border tics and labels might be
-			   hidden by the surface */
-#endif
   PM3D_IMPL_MODE implicit;
 			/* 1: [default] draw ALL surfaces with pm3d
 			   0: only surfaces specified with 'with pm3d' */
