@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: time.c,v 1.19.2.2 2009/06/12 05:05:44 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: time.c,v 1.20.2.2 2010/03/21 04:07:13 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - time.c */
@@ -559,6 +559,11 @@ xstrftime(
 		    return (0);
 		break;
 
+	    case 's':
+		CHECK_SPACE(12); /* large enough for year 9999 */
+		sprintf(s, "%.0f", gtimegm(tm));
+		break;
+
 	    case 'S':
 		FORMAT_STRING(1, 2, tm->tm_sec);	/* %02d */
 		break;
